package annas.graph.util.traversal;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;

import annas.graph.ArcInterface;
import annas.graph.GraphInterface;

/**
 * Performs a Breadth first traversal
 * 
 * @author Sam Wilson
 * 
 * @param <N>
 *            Node type
 * @param <A>
 *            Arc type
 */
public class BreadthFirst<N, A extends ArcInterface<N>> {
	
	private GraphInterface<N, A> g;
	
	private LinkedList<N> Order;
	
	private boolean Bipartite = true;

	/**
	 * 
	 */
	public BreadthFirst() {
		super();
		this.Order = new LinkedList<N>();
	}
	
	public BreadthFirst(GraphInterface<N, A> g) {
		super();
		this.g = g;
		this.Order = new LinkedList<N>();
	}
	
	public Iterator<N> run(N s) {
		this.BF(s);
		return this.Order.iterator();
	}
	
	public Iterator<N> run(N s, N tar) {
		this.run(s);
		this.BF(s,tar);
		return this.Order.iterator();
	}
	
	private void BF(N n) {
		Queue<N> l = new LinkedList<N>();
		HashMap<N, Integer> map = new HashMap<N, Integer>();
		map.put(n, 0);
		l.offer(n);
		
		while(l.size()>0){
			N m = l.poll();
			this.Order.add(m);
			ArrayList<A> arcs = this.g.getArc(m);
			for (int i = 0; i < arcs.size(); i++) {
				
				if (this.Order.contains(arcs.get(i).getHead())) {
				
				} else {
					l.offer(arcs.get(i).getHead());
					}
				
				if (!map.containsKey(arcs.get(i).getHead())) {
					map.put(arcs.get(i).getHead(), this.label(map.get(m)));

				} else {
					if (map.get(arcs.get(i).getHead()) == map.get(m)) {
						this.Bipartite = false;
					}
				}
			}
		}
		
	}
	
	private void BF(N n, N Target) {
		LinkedList<N> h = new LinkedList<N>();
		for(int i = 0; i<this.Order.size();i++){
			h.add(this.Order.get(i));
			if(h.get(h.size()-1).equals(Target)){
				this.Order = h;
				return;
			}
		}
	}
	

	public GraphInterface<N, A> getG() {
		return g;
	}

	public void setG(GraphInterface<N, A> g) {
		this.g = g;
	}
	
	private int label(int i){
		return (Math.abs(i-1));
	}

	@SuppressWarnings("hiding")
	class BFNode<N> {
		
		int label = Integer.MAX_VALUE;
		
		N n;

		
		public BFNode(N n, int label) {
			this.n = n;
			this.label = label;
		}

		public int getLabel() {
			return label;
		}

		public void setLabel(int label) {
			this.label = label;
		}

		public N getN() {
			return n;
		}

		public void setN(N n) {
			this.n = n;
		}
		
	}

	public boolean isBipartite() {
		return Bipartite;
	}

	public void setBipartite(boolean bipartite) {
		Bipartite = bipartite;
	}
}
