package clothSimulator;


/**
 * Home brew implementation of conjugate gradient
 * @author ben
 */
public class ConjugateGradientSolver implements LinearSolver {

    /** the size of the system */
    int n;
    
	/** 
	 * @return the type of the solve
	 */	
	@Override
	public String solverType() {
		return "ConjugateGradients";
	}

	private ArrayAsVector r, prev_r, prev_prev_r;
	private ArrayAsVector x, prev_x;
	private ArrayAsVector p, prev_p;
	private double alpha, beta;
	private ArrayAsVector temp_dofs, temp_constr;
	@Override
	public ArrayAsVector solveLinearSystem(ConstraintList constraintList, ArrayAsVector b) {
		if ( constraintList.theConstraints.size() != n ) {
		    r = new ArrayAsVector(constraintList.theConstraints.size());
		    prev_r = new ArrayAsVector(constraintList.theConstraints.size());
	        prev_prev_r = new ArrayAsVector(constraintList.theConstraints.size());	        
	        x = new ArrayAsVector(constraintList.theConstraints.size());
	        prev_x = new ArrayAsVector(constraintList.theConstraints.size());
	        p = new ArrayAsVector(constraintList.theConstraints.size());
	        prev_p = new ArrayAsVector(constraintList.theConstraints.size());
	        temp_constr = new ArrayAsVector(constraintList.theConstraints.size());
		}
		
		r.set(b);
		
		temp_dofs = new ArrayAsVector( constraintList.getNumDofs() );
		
		int k=0;
		while( !r.isZero() ){
			//r.set(new arrayAsVector(r.length()));
			k++;
			prev_prev_r.set(prev_r);
			prev_r.set(r);
			prev_p.set(p);
			prev_x.set(x);
			if(k==1) p.set(prev_r);
			else{
				beta=prev_r.inner(prev_r)/(prev_prev_r.inner(prev_prev_r));
				p.set(prev_p);
				p.scaleVector(beta);
				p.addVectors(prev_r);
			}
			//compute Ap
			temp_constr.set(p);
			constraintList.multGTrans( temp_constr, temp_dofs );
			constraintList.multMassMatrixInverse(temp_dofs);
			temp_constr=constraintList.multG(temp_dofs);
			//divide it into |prev_r|^2
			alpha=(prev_r.inner(prev_r))/(p.inner(temp_constr));
			//compute x;
			x.set(p);
			x.scaleVector(alpha);
			x.addVectors(prev_x);
			//finally compute r
			r.set(p);
			temp_dofs=constraintList.multGTrans(r);
			constraintList.multMassMatrixInverse(temp_dofs);
			r=constraintList.multG(temp_dofs);
			r.scaleVector(-alpha);
			r.addVectors(prev_r);	
			if(k>100){
				r.set(new ArrayAsVector(r.length()));
			}
		}
		
		
		/**here we return the solution to Ax=b which is just the vector of lagrange multipliers
		 * and we leave it up to the calling method to compute the forces.
		 */
		
		return x;
	}
	


}
